package com.jiuqi.util;

package .tmp;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

public final class SortedList implements List {
  private List items;
  
  private Comparator comparator;
  
  private int[] findStub = new int[1];
  
  public SortedList() {
    this.items = new ArrayList();
  }
  
  public SortedList(int initialCapacity) {
    this.items = new ArrayList(initialCapacity);
  }
  
  public SortedList(Comparator comparator) {
    this.items = new ArrayList();
    this.comparator = comparator;
  }
  
  public SortedList(int initalCapacity, Comparator comparator) {
    this.items = new ArrayList(initalCapacity);
    this.comparator = comparator;
  }
  
  public int compareObjects(Object o1, Object o2) {
    if (this.comparator != null)
      return this.comparator.compare(o1, o2); 
    if (o1 instanceof Comparable)
      return ((Comparable<Object>)o1).compareTo(o2); 
    if (o2 instanceof Comparable)
      return -((Comparable)o2).compareTo((T)o1); 
    throw new UnsupportedOperationException("对象比较时无法转化为Comparable接口！");
  }
  
  public boolean add(Object o) {
    int[] index = new int[1];
    find(o, index);
    this.items.add(index[0], o);
    return true;
  }
  
  public void add(int index, Object element) {
    add(element);
  }
  
  public void _add(int index, Object element) {
    this.items.add(index, element);
  }
  
  public boolean addAll(Collection<?> c) {
    List newItems = new ArrayList(c);
    reset(newItems);
    return true;
  }
  
  public boolean addAll(int index, Collection c) {
    return addAll(c);
  }
  
  public void clear() {
    this.items.clear();
  }
  
  public boolean contains(Object o) {
    return find(o, new int[1]);
  }
  
  public boolean containsAll(Collection c) {
    Iterator itr = c.iterator();
    while (itr.hasNext()) {
      if (!contains(itr.next()))
        return false; 
    } 
    return true;
  }
  
  public Object get(int index) {
    return this.items.get(index);
  }
  
  public int indexOf(Object o) {
    int low = 0;
    int high = size() - 1;
    while (low <= high) {
      int mid = (low + high) / 2;
      int flag = compareObjects(o, get(mid));
      if (flag == 0) {
        if (mid == low)
          return mid; 
        high = mid;
        continue;
      } 
      if (flag > 0) {
        low = mid + 1;
        continue;
      } 
      high = mid - 1;
    } 
    return -1;
  }
  
  public boolean isEmpty() {
    return this.items.isEmpty();
  }
  
  public Iterator iterator() {
    return this.items.iterator();
  }
  
  public int lastIndexOf(Object o) {
    int low = 0;
    int high = size() - 1;
    while (low <= high) {
      int mid = (low + high + 1) / 2;
      int flag = compareObjects(o, get(mid));
      if (flag == 0) {
        if (mid == high)
          return mid; 
        low = mid;
        continue;
      } 
      if (flag > 0) {
        low = mid + 1;
        continue;
      } 
      high = mid - 1;
    } 
    return -1;
  }
  
  public ListIterator listIterator() {
    return this.items.listIterator();
  }
  
  public ListIterator listIterator(int index) {
    return this.items.listIterator(index);
  }
  
  public Object remove(int index) {
    return this.items.remove(index);
  }
  
  public boolean remove(Object o) {
    return this.items.remove(o);
  }
  
  public boolean removeAll(Collection c) {
    boolean ret = false;
    Iterator itr = c.iterator();
    while (itr.hasNext()) {
      if (remove(itr.next()))
        ret = true; 
    } 
    return ret;
  }
  
  public boolean retainAll(Collection c) {
    List list = new com.jiuqi.util.SortedList(c.size(), this.comparator);
    list.addAll(c);
    boolean ret = false;
    for (int i = size() - 1; i >= 0; i--) {
      if (!list.contains(get(i))) {
        remove(i);
        ret = true;
      } 
    } 
    return ret;
  }
  
  public Object set(int index, Object element) {
    if (compareObjects(get(index), element) == 0)
      return this.items.set(index, element); 
    Object ret = this.items.get(index);
    this.items.remove(index);
    add(element);
    return ret;
  }
  
  public int size() {
    return this.items.size();
  }
  
  public List subList(int fromIndex, int toIndex) {
    com.jiuqi.util.SortedList result = new com.jiuqi.util.SortedList(this.comparator);
    result.items = this.items.subList(fromIndex, toIndex);
    return result;
  }
  
  public Object[] toArray() {
    return this.items.toArray();
  }
  
  public Object[] toArray(Object[] a) {
    return this.items.toArray(a);
  }
  
  public Comparator getComparator() {
    return this.comparator;
  }
  
  public void setComparator(Comparator comparator) {
    this.comparator = comparator;
  }
  
  public boolean find(Object element, int[] index) {
    int low = 0;
    int high = size() - 1;
    while (low <= high) {
      int mid = (low + high) / 2;
      int flag = compareObjects(element, get(mid));
      if (flag == 0) {
        index[0] = mid;
        return true;
      } 
      if (flag > 0) {
        low = mid + 1;
        continue;
      } 
      high = mid - 1;
    } 
    index[0] = low;
    return false;
  }
  
  public int find(Object element) {
    if (find(element, this.findStub))
      return this.findStub[0]; 
    return -1;
  }
  
  public void reset(List<?> items) {
    if (items == null)
      throw new NullPointerException("items"); 
    Collections.sort(items, (Comparator<?>)new Object(this));
    this.items = items;
  }
}
